@@ -5,7 +5,7 @@ import tkinter as tk
 from tkinter import *
 from tkinter.ttk import *
 from funing.ui.main_ui import MainUI
-from langcodes import Language
+# from langcodes import Language
 import gettext
 import sys
 import os
@@ -23,6 +23,16 @@ from funing._ui import error
 import numpy as np
 from cv2 import haarcascades
 import webbrowser
+from enum import Enum
+
+class SourceType( Enum ):
+    NULL    =   0
+    IMG     =   1
+    VID     =   2
+
+class FDoing( Enum ):
+    REC     =   0
+    PICK    =   1
 
 class _MainUI():
     def __init__(self):
@@ -30,8 +40,6 @@ class _MainUI():
         self.mainui.place()
         self.source = -1
         self.root_after = -1
-        # face num for face_label
-        self.lang_code = settings.lang_code
         self.fxfy = None
         self.image_exts = ['jpg','png', 'jpeg', 'webp']
         self.video_exts = ['mp4','avi','3gp','webm','mkv']
@@ -42,11 +50,11 @@ class _MainUI():
         self.about_tl = None
         # vid
         self.vid = None
-        self.vid_width = 0
-        self.vid_height = 0
+        self.frame_width = 0
+        self.frame_height = 0
         self.vid_fps = 0
 
-        self.source_type = -1 # 0-> img , 1-> video
+        self.source_type = SourceType.NULL 
 
         self.cur_frame = None
         self.face_rects = []
@@ -57,11 +65,10 @@ class _MainUI():
         self.save_size = (100,100)
         self.zoomed_in_face_label = (0,0)
         
-        self.doing = 'p'  # 'p'->'pick', r->'rec'
+        self.fdoing = FDoing.PICK 
 
-        self.pause = False
+        self.paused = False
         self.face_frames = []
-        self.curf_index = 0
         # rec_result
         self.rec_gray_img = None
         # rec_faces
@@ -73,16 +80,15 @@ class _MainUI():
         self.hff_xml_path = os.path.join( haarcascades ,\
          "haarcascade_frontalface_default.xml" )
         self.recognizer=cv2.face.EigenFaceRecognizer_create()
-        self.face_casecade=cv2.CascadeClassifier( self.hff_xml_path )   
-        self.face_enter_count = settings.face_enter_count
+        self.face_casecade=cv2.CascadeClassifier( self.hff_xml_path )
         #screen
         try:self.screenwidth = self.mainui.root.winfo_screenwidth();\
             self.screenheight = self.mainui.root.winfo_screenheight()
         except: print(_('No desktop environment is detected! ')); exit()  
         if not settings.data_empty():
-            self.recognizer_train()           
+            self.recognizer_train()
         self.set_ui_events()
-        self.mainui.mainloop()    
+        self.mainui.mainloop()
 
     def load_images( self ):
         '''
@@ -135,8 +141,8 @@ class _MainUI():
     def open_vid_cap( self ):
         self.vid = cv2.VideoCapture( self.source )
         if not self.vid.isOpened(): self.show_nsrc_error(); return
-        self.vid_width = self.vid.get(cv2.CAP_PROP_FRAME_WIDTH)
-        self.vid_height = self.vid.get(cv2.CAP_PROP_FRAME_HEIGHT)
+        self.frame_width = self.vid.get(cv2.CAP_PROP_FRAME_WIDTH)
+        self.frame_height = self.vid.get(cv2.CAP_PROP_FRAME_HEIGHT)
         self.vid_fps = self.vid.get(cv2.CAP_PROP_FPS)
     
     def close_vid_cap( self ):
@@ -145,16 +151,11 @@ class _MainUI():
         self.vid= None  
     
     def set_ui_events( self ):
-        self.rbmixfm.lang_combobox.bind('<<ComboboxSelected>>',
-            self.change_language )
-        # self.showfm.ct_entry.bind('<FocusOut>', None )
         self.showfm.pp_btn['command'] = self.pause_play
         self.showfm.rec_btn['command'] = self.recf_v0
         self.showfm.pick_btn['command'] = self.pick_v0
         self.showfm.showf_go_btn['command'] = self.show_go
         self.showfm.showf_optionmenu_sv.trace('w', self.show_from )
-        # self.infofm.prevf_btn['command'] = self.prevf
-        # self.infofm.nextf_btn['command'] = self.nextf
         self.infofm.save_btn['command'] = self.savef
         self.rbmixfm.about_fn_btn['command'] = self.about_fn
         self.mainui.root.protocol("WM_DELETE_WINDOW", self.destroy )
@@ -164,50 +165,28 @@ class _MainUI():
         exit()
 
     def pause_play( self, *args ):
-        if self.source_type != 1 : return
-        if self.pause: 
-            self.pause = False
+        if self.source_type != SourceType.VID : return
+        if self.paused: 
+            self.paused = False
             self.refresh_frame()
             self.showfm.pp_sv.set( _('Pause') )
-            if settings.debug:
+            if settings.debug():
                 print( 'Play. . .' )
             
         else:
-            self.root_after_cancel()
+            self.cancel_root_after()
             self.showfm.pp_sv.set( _('Play') )
-            self.pause = True
-            if settings.debug:
+            self.paused = True
+            if settings.debug():
                 print( 'Pause. . .' )
 
-    def pick(self):
-        if self.vid is None: return
-        count = 0
-        self.cur_info_id = str(uuid.uuid4())
-        self.recfs = []
-        self.face_frames = []
-        while(True):
-            ret, self.frame=self.vid.read()
-            if ret:
-                gray_img=cv2.cvtColor(self.frame,cv2.COLOR_BGR2GRAY)
-                faces = self.face_casecade.detectMultiScale(gray_img,1.3,5)
-                if len( faces ) < 1:continue
-                x,y,w,h = faces[0]
-                new_frame=cv2.resize( self.frame[y:y+h,x:x+w], (92,112),\
-                    interpolation=cv2.INTER_LINEAR)
-                self.face_frames.append( new_frame )
-                count+=1
-                if count > self.face_enter_count: break
-        
-        self.infofm.face_text.delete(1.0,END)
-        self.change_face_show(0)
-
     def pick_v0(self):
         
-        if self.doing == 'r':
+        if self.fdoing == FDoing.REC:
             self.clear_faces_frame()
-            self.doing ='p'
+            self.fdoing = FDoing.PICK # 'p'
 
-        if (not self.pause) and self.vid :
+        if (not self.paused) and self.vid :
             _, self.cur_frame = self.vid.read()
             
         if self.cur_frame is None: return
@@ -219,7 +198,7 @@ class _MainUI():
         gray_img=cv2.cvtColor(self.cur_frame,cv2.COLOR_BGR2GRAY)
         self.face_rects = self.face_casecade.detectMultiScale(gray_img,1.3,5)
         
-        if settings.debug:
+        if settings.debug():
             print( self.face_rects )
             print( type( self.face_rects ) )
 
@@ -229,8 +208,7 @@ class _MainUI():
     def clear_face_text( self ):
         self.infofm.face_text.delete(1.0,END)
 
-    def add_face_label_p(self, num):
-        
+    def add_face_label_p(self, num):        
         x,y,w,h = self.face_rects[ num ]
         _w = max( w, h )
         
@@ -253,14 +231,14 @@ class _MainUI():
     def del_face_label_p( self, e, num):
         del self.picked_face_frames[ num ]
         e.widget.destroy()
-        if settings.debug:
+        if settings.debug():
             print( len(self.picked_face_frames) )
 
     def show_go( self, *args ):
         self.showf_sv = self.showfm.showf_sv.get()
         if len( self.showf_sv.strip() ) < 1: return
         self.rec_img = False
-        self.root_after_cancel()
+        self.cancel_root_after()
         showf_ext = self.showf_sv.split('.')[-1]
         if showf_ext in self.video_exts:
             self.source = self.showf_sv
@@ -304,26 +282,23 @@ class _MainUI():
             self.showfm.showf_sv.set( self.source )
             self.play_video()
     
-    def root_after_cancel( self ):
+    def cancel_root_after( self ):
         if self.root_after != -1:
-            
-            if self.vid is not None and self.vid.isOpened():
-                _, self.cur_frame = self.vid.read()
-
             self.mainui.root.after_cancel( self.root_after )
             self.close_vid_cap()
             self.vid = None
         
     def play_video( self ):
-        self.pause = False
-        self.source_type = 1
+        self.paused = False
+        self.source_type = SourceType.VID
         self.close_vid_cap()
         self.open_vid_cap()
         self.get_vid_resize_fxfy()
         self.refresh_frame()
     
     def refresh_frame(self):
-        if self.source_type == 0:return
+        if self.source_type != SourceType.VID: return
+        
         if self.vid == None: self.vid = cv2.VideoCapture( self.source )
         if not self.vid.isOpened(): self.show_nsrc_error(); return
 
@@ -345,7 +320,36 @@ class _MainUI():
         self.showfm.vid_frame_label.imgtk = imgtk
         self.showfm.vid_frame_label.configure(image=imgtk)
 
-        if not self.pause:
+        if self.paused:
+            self.cur_frame = frame
+        else:
+            self.root_after = self.mainui.root.after( \
+                int(1000/self.vid_fps) , self.refresh_frame )
+
+    def refresh_frame_v0(self):
+        if self.source_type != SourceType.VID :return
+        if self.vid == None: self.vid = cv2.VideoCapture( self.source )
+        if not self.vid.isOpened(): self.show_nsrc_error(); return
+
+        _, frame = self.vid.read()
+
+        gray_img=cv2.cvtColor( frame,cv2.COLOR_BGR2GRAY)
+        rects = self.face_casecade.detectMultiScale(gray_img,1.3,5)
+
+        for (x,y,w,h) in rects:
+            frame=cv2.rectangle( frame,(x,y),(x+w,y+h),(255,0,0),2)  
+        rects = None
+
+        vid_img = cv2.resize( frame , (0,0) , \
+            fx = self.fxfy, fy = self.fxfy )
+
+        vid_img = cv2.cvtColor( vid_img, cv2.COLOR_BGR2RGB )
+        vid_img = Image.fromarray( vid_img )
+        imgtk = ImageTk.PhotoImage( image=vid_img )
+        self.showfm.vid_frame_label.imgtk = imgtk
+        self.showfm.vid_frame_label.configure(image=imgtk)
+
+        if not self.paused:
             self.root_after = self.mainui.root.after( \
                 int(1000/self.vid_fps) , self.refresh_frame )
 
@@ -355,23 +359,20 @@ class _MainUI():
         h = self.screenheight/2
         r = w/h 
         
-        self.vid_width, self.vid_height, _ = self.cur_frame.shape
-
-        r0 = self.vid_width/self.vid_height
+        r0 = self.frame_width/self.frame_height
         r1= r0/r 
-        self.fxfy = h/self.vid_height if r1<r else w/self.vid_width
-        if settings.debug:
+        self.fxfy = h/self.frame_height if r1<r else w/self.frame_width
+        if settings.debug():
             print('self.fxfy: ', self.fxfy)
         
 
     def view_image( self ):
-        self.source_type = 0
-        self.root_after_cancel()
+        self.source_type = SourceType.IMG
+        self.cancel_root_after()
 
         self.cur_frame  = cv2.imread( self.face_src_path )
 
-        if settings.debug:
-            print( self.cur_frame.shape,self.cur_frame.size )
+        self.frame_width, self.frame_height, _ = self.cur_frame.shape
             
         self.get_img_resize_fxfy()
         
@@ -404,112 +405,39 @@ class _MainUI():
         self.showfm.vid_frame_label.configure(image=imgtk)
 
     def get_vid_resize_fxfy( self ):
-        if self.vid_width == self.vid_height == 0: 
+        if self.frame_width == self.frame_height == 0: 
             if debug: print('self.iru is None')
             return
         w = self.screenwidth/2
         h = self.screenheight/2
         r = w/h 
-        r0 = self.vid_width/self.vid_height
+        r0 = self.frame_width/self.frame_height
         r1= r0/r 
-        self.fxfy = h/self.vid_height if r1<r else w/self.vid_width
-        if settings.debug:
+        self.fxfy = h/self.frame_height if r1<r else w/self.frame_width
+        if settings.debug():
             print('self.fxfy: ', self.fxfy)
 
     def show_nsrc_error( self ):
         unable_open_s = _('Unable to open video source')
         messagebox.showerror( unable_open_s, unable_open_s+': '+self.showf_sv )
-
-    def prevf( self ):
-        self.change_face_show(-1)
-        pass
-
-    def nextf( self ):
-        self.change_face_show(+1)
-        pass
     
     def savef( self ):
         if self.cur_info_id == None: return
         info =self.infofm.face_text.get("1.0", "end-1c")
         info_file_path = os.path.join( settings.infos_path, self.cur_info_id )
         open( info_file_path, 'w+' ).write( info )
-        img_path =os.path.join( settings.faces_path , self.cur_info_id )
-        os.makedirs(img_path,exist_ok=True )
-        count = 0
-        for f in self.picked_face_frames:
-            cv2.imwrite( f'{img_path}/{count}.png' , f)
-            count+=1
-        self.cur_info_id = None
-        if settings.debug:print( 'info > ' + info )
-        self.recognizer_train()
-    
-    def update_num_label(self):
-        self.infofm.num_label['text'] = \
-        f'{self.curf_index+1}/{len(self.face_frames)+len(self.recfs)-1}'
-            
-    def change_face_show(self, _as, rec = True):
-        if len(self.face_frames) >0:
-            # NEW
-            self.curf_index += _as
-            self.curf_index = 0 if self.curf_index < 0 else self.face_enter_count-1\
-            if self.curf_index >= self.face_enter_count else self.curf_index 
-
-            vid_img = cv2.cvtColor( self.face_frames[ self.curf_index ], \
-            cv2.COLOR_BGR2RGB )
-            vid_img = Image.fromarray( vid_img )
-            imgtk = ImageTk.PhotoImage( image=vid_img )
-            self.infofm.curf_label.imgtk = imgtk
-            self.infofm.curf_label.configure(image=imgtk)
-            self.update_num_label()
-
-        elif len(self.recfs) > 0:
-            # RECOGNIZE
-            _len = len( self.recfs )
-            self.curf_index+=_as
-            self.curf_index = 0 if self.curf_index < 0 else _len - 1\
-            if self.curf_index >= _len else self.curf_index 
-
-            x,y,w,h = self.recfs[ self.curf_index ]
-            roi_gray= self.rec_gray_img[y:y+h,x:x+w]
-            roi_gray= cv2.resize( roi_gray, (92,112),\
-            interpolation=cv2.INTER_LINEAR)
-            result=self.recognizer.predict(roi_gray)
-            _id = self.info_ids[result[0]]
-            _h = h if h>w else w
-            frame = self.cur_frame[y:y+_h,x:x+_h]
-            frame = cv2.resize(frame, (200,200))
-
-            vid_img = cv2.cvtColor( frame, cv2.COLOR_BGR2RGB )
-            vid_img = Image.fromarray( vid_img  )
-            imgtk = ImageTk.PhotoImage( image=vid_img )
-            self.infofm.curf_label.imgtk = imgtk
-            self.infofm.curf_label.configure(image=imgtk)
-
-            info_file_path = os.path.join( \
-            settings.infos_path,  _id )
-            self.infofm.face_text.delete(1.0,END)
-            self.infofm.face_text.insert('1.0', \
-            open( info_file_path, 'r' ).read() )
-            self.update_num_label()
-    
-    def recf(self):
-        self.face_frames = []
-        self.recfs = []
-        if settings.debug: 
-            print('self.cur_frame not None')
-        self.rec_gray_img=cv2.cvtColor(self.cur_frame,cv2.COLOR_BGR2GRAY)
-        self.face_rects=self.face_casecade.detectMultiScale(\
-        self.rec_gray_img,1.3,5)
-        if len( self.recfs ) < 1: return        
+        if self.fdoing == FDoing.PICK:
+            img_path =os.path.join( settings.faces_path , self.cur_info_id )
+            os.makedirs(img_path,exist_ok=True )
+            count = 0
+            for f in self.picked_face_frames:
+                cv2.imwrite( f'{img_path}/{count}.png' , f)
+                count+=1
+            self.cur_info_id = None
         
-        self.change_face_show(0)
-
-        for (x,y,w,h) in self.recfs:
-            self.cur_frame=cv2.rectangle(\
-            self.cur_frame,(x,y),(x+w,y+h),(255,0,0),2)  
-
-        self.cur_frame2label()
-    
+        if settings.debug():print( 'info > ' + info )
+        self.recognizer_train()
+                
     def restore_face_label_size( self, index ):
         label, index = self.zoomed_in_face_label
         if not label.winfo_exists(): return
@@ -524,7 +452,7 @@ class _MainUI():
         label.configure(image=imgtk)
 
 
-    def show_info( self, label, _id, index):
+    def show_info( self, label,  index):
 
         if (self.zoomed_in_face_label[0]!=0) and \
         (self.zoomed_in_face_label[0] != label) :
@@ -542,7 +470,7 @@ class _MainUI():
         self.zoomed_in_face_label = (label, index)
 
         info_file_path = os.path.join( \
-        settings.infos_path,  _id )
+        settings.infos_path,  self.cur_info_id )
         self.infofm.face_text.delete(1.0,END)
         
         if not os.path.exists( info_file_path ): 
@@ -553,7 +481,6 @@ class _MainUI():
 
 
     def add_face_label_r( self, num ):
-
         index = len( self.showed_face_frames )
 
         new_fl = Label( self.infofm.faces_frame )
@@ -563,9 +490,9 @@ class _MainUI():
         roi_gray= cv2.resize( roi_gray, self.save_size,\
         interpolation=cv2.INTER_LINEAR)
     
-        result=self.recognizer.predict(roi_gray)
+        label, confidence=self.recognizer.predict(roi_gray)
 
-        _id = self.info_ids[result[0]]
+        self.cur_info_id = self.info_ids[ label ]
         _h = max( h, w )
         frame = self.cur_frame[y:y+_h,x:x+_h]
         frame = cv2.resize(frame, self.show_size )
@@ -581,11 +508,11 @@ class _MainUI():
         new_fl.bind("<Double-Button-1>",lambda e: \
         self.del_face_label_r(e, index))
 
-        new_fl.bind("<Button-1>",lambda e: self.show_info(new_fl , _id, index) )
+        new_fl.bind("<Button-1>",lambda e: self.show_info(new_fl ,  index) )
 
         new_fl.pack(side=LEFT)
 
-        self.show_info(new_fl , _id, index)
+        self.show_info(new_fl ,  index)
 
              
     def del_face_label_r( self, e, index):
@@ -593,25 +520,26 @@ class _MainUI():
             self.zoomed_in_face_label= (0,0)
         e.widget.destroy()
         self.showed_face_frames[index] = None
+        self.infofm.face_text.delete(1.0,END)
     
     def clear_faces_frame( self ):
         for child in self.infofm.faces_frame.winfo_children():
             child.destroy()
 
-
     def recf_v0(self):
 
-        if self.source_type == -1:return
-        if (not self.pause) and self.vid:
+        if self.source_type == SourceType.NULL: return
+
+        if (not self.paused) and self.vid:
             _, self.cur_frame = self.vid.read()
         
         if settings.data_empty():
             self.show_data_empty()
             return
         
-        if self.doing == 'p':
+        if self.fdoing == FDoing.PICK:
             self.clear_faces_frame()
-            self.doing ='r'
+            self.fdoing = FDoing.REC
 
         if self.cur_frame is None: return
         
@@ -622,38 +550,17 @@ class _MainUI():
         self.rec_gray_img,1.3,5)
 
         if len( self.face_rects ) < 1: 
-            if settings.debug:
+            if settings.debug():
                 print('len( self.face_rects ) < 1 ')
             return
 
         for i in range( len(self.face_rects)) :
             self.add_face_label_r(i)
-            
-             
-    def change_language(self, lang ):
-
-        lang_display_name = self.rbmixfm.lang_combobox_var.get()
-        new_lang_code = Language.find( lang_display_name ).to_tag()
-        new_lang_code.replace('-','_')
-        if settings.debug:
-            print( 'new_lang_code: ', new_lang_code, \
-            'lang_code: ', settings.lang_code )
-
-        if new_lang_code == settings.lang_code: return
-
-        restartapp = messagebox.askyesno(
-            title = _('Restart Funing Now?')
-        )
-        if restartapp:
-            settings.config_yml['lang_code'] = new_lang_code
-            yaml.dump( settings.config_yml, open( settings._config_path, 'w') )
-            sys_executable = sys.executable
-            os.execl(sys_executable, sys_executable, * sys.argv)
-        pass
-
+                         
     def show_nsrc_error( self ):
         unable_open_s = _('Unable to open video source')
         messagebox.showerror(  unable_open_s, unable_open_s+': '+self.showf_sv )
+
     def show_data_empty( self ):
         unable_open_s = _('Nothing enter')
         msg  = _("You haven't entered anything yet! ")
